@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for anomaly detectors' REST calls
 
-   https://bigml.com/developers/anomalies
+   https://bigml.com/api/anomalies
 
 """
 
@@ -30,7 +30,7 @@ except ImportError:
 from bigml.resourcehandler import ResourceHandler
 from bigml.resourcehandler import (check_resource_type, resource_is_ready,
                                    get_anomaly_id)
-from bigml.resourcehandler import ANOMALY_PATH, LOGGER
+from bigml.constants import ANOMALY_PATH
 
 
 class AnomalyHandler(ResourceHandler):
