@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for anomaly scores' REST calls
 
-   https://bigml.com/developers/anomalyscores
+   https://bigml.com/api/anomalyscores
 
 """
 
@@ -31,8 +31,8 @@ from bigml.resourcehandler import ResourceHandler
 from bigml.resourcehandler import (check_resource_type, get_resource_type,
                                    check_resource,
                                    get_anomaly_score_id, get_anomaly_id)
-from bigml.resourcehandler import (ANOMALY_SCORE_PATH, ANOMALY_PATH,
-                                   TINY_RESOURCE, LOGGER)
+from bigml.constants import (ANOMALY_SCORE_PATH, ANOMALY_PATH,
+                             TINY_RESOURCE)
 
 
 class AnomalyScoreHandler(ResourceHandler):
@@ -48,7 +48,7 @@ class AnomalyScoreHandler(ResourceHandler):
            instantiated independently.
 
         """
-        self.anomaly_score_url = self.url + ANOMALY_SCORE_PATH
+        self.anomaly_score_url = self.prediction_base_url + ANOMALY_SCORE_PATH
 
     def create_anomaly_score(self, anomaly, input_data=None,
                              args=None, wait_time=3, retries=10):
@@ -81,7 +81,7 @@ class AnomalyScoreHandler(ResourceHandler):
         return self._create(self.anomaly_score_url, body,
                             verify=self.verify)
 
-    def get_anomaly_score(self, anomaly_score):
+    def get_anomaly_score(self, anomaly_score, query_string=''):
         """Retrieves an anomaly score.
 
         """
@@ -89,7 +89,8 @@ class AnomalyScoreHandler(ResourceHandler):
                             message="An anomaly score id is needed.")
         anomaly_score_id = get_anomaly_score_id(anomaly_score)
         if anomaly_score_id:
-            return self._get("%s%s" % (self.url, anomaly_score_id))
+            return self._get("%s%s" % (self.url, anomaly_score_id),
+                             query_string=query_string)
 
     def list_anomaly_scores(self, query_string=''):
         """Lists all your anomaly_scores.
@@ -117,4 +118,3 @@ class AnomalyScoreHandler(ResourceHandler):
         anomaly_score_id = get_anomaly_score_id(anomaly_score)
         if anomaly_score_id:
             return self._delete("%s%s" % (self.url, anomaly_score_id))
-
