@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for batch centroids' REST calls
 
-   https://bigml.com/developers/batchcentroids
+   https://bigml.com/api/batchcentroids
 
 """
 
@@ -30,7 +30,7 @@ from bigml.bigmlconnection import DOWNLOAD_DIR
 from bigml.resourcehandler import ResourceHandler
 from bigml.resourcehandler import (check_resource_type,
                                    get_batch_centroid_id)
-from bigml.resourcehandler import BATCH_CENTROID_PATH, CLUSTER_PATH
+from bigml.constants import BATCH_CENTROID_PATH, CLUSTER_PATH
 
 
 class BatchCentroidHandler(ResourceHandler):
@@ -46,7 +46,8 @@ class BatchCentroidHandler(ResourceHandler):
            instantiated independently.
 
         """
-        self.batch_centroid_url = self.url + BATCH_CENTROID_PATH
+        self.batch_centroid_url = self.prediction_base_url \
+            + BATCH_CENTROID_PATH
 
     def create_batch_centroid(self, cluster, dataset,
                               args=None, wait_time=3, retries=10):
@@ -66,7 +67,7 @@ class BatchCentroidHandler(ResourceHandler):
             body = json.dumps(create_args)
             return self._create(self.batch_centroid_url, body)
 
-    def get_batch_centroid(self, batch_centroid):
+    def get_batch_centroid(self, batch_centroid, query_string=''):
         """Retrieves a batch centroid.
 
            The batch_centroid parameter should be a string containing the
@@ -80,7 +81,8 @@ class BatchCentroidHandler(ResourceHandler):
                             message="A batch centroid id is needed.")
         batch_centroid_id = get_batch_centroid_id(batch_centroid)
         if batch_centroid_id:
-            return self._get("%s%s" % (self.url, batch_centroid_id))
+            return self._get("%s%s" % (self.url, batch_centroid_id),
+                             query_string=query_string)
 
     def download_batch_centroid(self, batch_centroid, filename=None):
         """Retrieves the batch centroid file.
