@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for batch predictions' REST calls
 
-   https://bigml.com/developers/batchpredictions
+   https://bigml.com/api/batchpredictions
 
 """
 
@@ -30,8 +30,7 @@ from bigml.bigmlconnection import DOWNLOAD_DIR
 from bigml.resourcehandler import ResourceHandler
 from bigml.resourcehandler import (check_resource_type,
                                    get_batch_prediction_id)
-from bigml.resourcehandler import (BATCH_PREDICTION_PATH, ENSEMBLE_PATH,
-                                   MODEL_PATH)
+from bigml.constants import BATCH_PREDICTION_PATH, SUPERVISED_PATHS
 
 
 class BatchPredictionHandler(ResourceHandler):
@@ -47,7 +46,8 @@ class BatchPredictionHandler(ResourceHandler):
            instantiated independently.
 
         """
-        self.batch_prediction_url = self.url + BATCH_PREDICTION_PATH
+        self.batch_prediction_url = (self.prediction_base_url +
+                                     BATCH_PREDICTION_PATH)
 
     def create_batch_prediction(self, model, dataset,
                                 args=None, wait_time=3, retries=10):
@@ -62,15 +62,14 @@ class BatchPredictionHandler(ResourceHandler):
         if args is not None:
             create_args.update(args)
 
-        model_types = [ENSEMBLE_PATH, MODEL_PATH]
         origin_resources_checked = self.check_origins(
-            dataset, model, create_args, model_types=model_types,
+            dataset, model, create_args, model_types=SUPERVISED_PATHS,
             wait_time=wait_time, retries=retries)
         if origin_resources_checked:
             body = json.dumps(create_args)
             return self._create(self.batch_prediction_url, body)
 
-    def get_batch_prediction(self, batch_prediction):
+    def get_batch_prediction(self, batch_prediction, query_string=''):
         """Retrieves a batch prediction.
 
            The batch_prediction parameter should be a string containing the
@@ -84,7 +83,8 @@ class BatchPredictionHandler(ResourceHandler):
                             message="A batch prediction id is needed.")
         batch_prediction_id = get_batch_prediction_id(batch_prediction)
         if batch_prediction_id:
-            return self._get("%s%s" % (self.url, batch_prediction_id))
+            return self._get("%s%s" % (self.url, batch_prediction_id),
+                             query_string=query_string)
 
     def download_batch_prediction(self, batch_prediction, filename=None):
         """Retrieves the batch predictions file.
