@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for centroids' REST calls
 
-   https://bigml.com/developers/centroids
+   https://bigml.com/api/centroids
 
 """
 
@@ -31,8 +31,7 @@ from bigml.resourcehandler import ResourceHandler
 from bigml.resourcehandler import (check_resource_type, get_resource_type,
                                    check_resource,
                                    get_centroid_id, get_cluster_id)
-from bigml.resourcehandler import (CENTROID_PATH, CLUSTER_PATH, TINY_RESOURCE,
-                                   LOGGER)
+from bigml.constants import CENTROID_PATH, CLUSTER_PATH, TINY_RESOURCE
 
 
 class CentroidHandler(ResourceHandler):
@@ -48,7 +47,7 @@ class CentroidHandler(ResourceHandler):
            instantiated independently.
 
         """
-        self.centroid_url = self.url + CENTROID_PATH
+        self.centroid_url = self.prediction_base_url + CENTROID_PATH
 
     def create_centroid(self, cluster, input_data=None,
                         args=None, wait_time=3, retries=10):
@@ -81,7 +80,7 @@ class CentroidHandler(ResourceHandler):
         return self._create(self.centroid_url, body,
                             verify=self.verify)
 
-    def get_centroid(self, centroid):
+    def get_centroid(self, centroid, query_string=''):
         """Retrieves a centroid.
 
         """
@@ -89,7 +88,8 @@ class CentroidHandler(ResourceHandler):
                             message="A centroid id is needed.")
         centroid_id = get_centroid_id(centroid)
         if centroid_id:
-            return self._get("%s%s" % (self.url, centroid_id))
+            return self._get("%s%s" % (self.url, centroid_id),
+                             query_string=query_string)
 
     def list_centroids(self, query_string=''):
         """Lists all your centroids.
