@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for datasets' REST calls
 
-   https://bigml.com/developers/datasets
+   https://bigml.com/api/datasets
 
 """
 
@@ -33,8 +33,8 @@ from bigml.resourcehandler import (check_resource_type,
                                    get_resource_type, resource_is_ready,
                                    check_resource, get_source_id,
                                    get_dataset_id, get_cluster_id)
-from bigml.resourcehandler import (DATASET_PATH, SOURCE_PATH,
-                                   TINY_RESOURCE, CLUSTER_PATH, LOGGER)
+from bigml.constants import (DATASET_PATH, SOURCE_PATH,
+                             TINY_RESOURCE, CLUSTER_PATH)
 
 
 class DatasetHandler(ResourceHandler):
@@ -105,7 +105,7 @@ class DatasetHandler(ResourceHandler):
                                          wait_time=wait_time,
                                          retries=retries,
                                          raise_on_error=True, api=self)
-                if not 'centroid' in create_args:
+                if 'centroid' not in create_args:
                     try:
                         centroid = cluster['object'][
                             'cluster_datasets_ids'].keys()[0]
@@ -187,7 +187,7 @@ class DatasetHandler(ResourceHandler):
 
         """
         errors_dict = {}
-        if not isinstance(dataset, dict) or not 'object' in dataset:
+        if not isinstance(dataset, dict) or 'object' not in dataset:
             check_resource_type(dataset, DATASET_PATH,
                                 message="A dataset id is needed.")
             dataset_id = get_dataset_id(dataset)
