@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for ensembles' REST calls
 
-   https://bigml.com/developers/ensembles
+   https://bigml.com/api/ensembles
 
 """
 
@@ -30,7 +30,7 @@ except ImportError:
 from bigml.resourcehandler import ResourceHandler
 from bigml.resourcehandler import (check_resource_type, resource_is_ready,
                                    get_ensemble_id)
-from bigml.resourcehandler import ENSEMBLE_PATH, LOGGER
+from bigml.constants import ENSEMBLE_PATH
 
 
 class EnsembleHandler(ResourceHandler):
