@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2012-2015 BigML
+# Copyright 2012-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -37,16 +37,19 @@ model.predict({"petal length": 3, "petal width": 1})
 
 """
 import logging
-LOGGER = logging.getLogger('BigML')
-
-
-import csv
 import ast
-from bigml.model import Model
+
+
+from bigml.model import Model, cast_prediction
 from bigml.model import LAST_PREDICTION
 from bigml.util import get_predictions_file_name
 from bigml.multivote import MultiVote
-from bigml.multivote import PLURALITY_CODE
+from bigml.multivote import PLURALITY_CODE, CONFIDENCE_CODE, PROBABILITY_CODE
+from bigml.multivotelist import MultiVoteList
+from bigml.io import UnicodeWriter, UnicodeReader
+
+
+LOGGER = logging.getLogger('BigML')
 
 
 def read_votes(votes_files, to_prediction, data_locale=None):
@@ -70,23 +73,24 @@ def read_votes(votes_files, to_prediction, data_locale=None):
     for order in range(0, len(votes_files)):
         votes_file = votes_files[order]
         index = 0
-        for row in csv.reader(open(votes_file, "U"), lineterminator="\n"):
-            prediction = to_prediction(row[0], data_locale=data_locale)
-            if index > (len(votes) - 1):
-                votes.append(MultiVote([]))
-            distribution = None
-            instances = None
-            if len(row) > 2:
-                distribution = ast.literal_eval(row[2])
-                instances = int(row[3])
-                try:
-                    confidence = float(row[1])
-                except ValueError:
-                    confidence = 0
-            prediction_row = [prediction, confidence, order,
-                              distribution, instances]
-            votes[index].append_row(prediction_row)
-            index += 1
+        with UnicodeReader(votes_file) as rdr:
+            for row in rdr:
+                prediction = to_prediction(row[0], data_locale=data_locale)
+                if index > (len(votes) - 1):
+                    votes.append(MultiVote([]))
+                distribution = None
+                instances = None
+                if len(row) > 2:
+                    distribution = ast.literal_eval(row[2])
+                    instances = int(row[3])
+                    try:
+                        confidence = float(row[1])
+                    except ValueError:
+                        confidence = 0.0
+                prediction_row = [prediction, confidence, order,
+                                  distribution, instances]
+                votes[index].append_row(prediction_row)
+                index += 1
     return votes
 
 
@@ -98,13 +102,18 @@ class MultiModel(object):
 
     """
 
-    def __init__(self, models, api=None):
+    def __init__(self, models, api=None, fields=None, class_names=None):
         self.models = []
+        self.class_names = class_names
+
         if isinstance(models, list):
-            for model in models:
-                self.models.append(Model(model, api=api))
+            if all([isinstance(model, Model) for model in models]):
+                self.models = models
+            else:
+                for model in models:
+                    self.models.append(Model(model, api=api, fields=fields))
         else:
-            self.models.append(Model(models, api=api))
+            self.models.append(Model(models, api=api, fields=fields))
 
     def list_models(self):
         """Lists all the model/ids that compound the multi model.
@@ -112,9 +121,8 @@ class MultiModel(object):
         """
         return [model.resource() for model in self.models]
 
-    def predict(self, input_data, by_name=True, method=PLURALITY_CODE,
-                with_confidence=False, options=None,
-                missing_strategy=LAST_PREDICTION):
+    def predict(self, input_data, method=PLURALITY_CODE, options=None,
+                missing_strategy=LAST_PREDICTION, full=False):
         """Makes a prediction based on the prediction made by every model.
 
            The method parameter is a numeric key to the following combination
@@ -125,15 +133,25 @@ class MultiModel(object):
               2 - probability weighted majority vote / average:
                   PROBABILITY_CODE
               3 - threshold filtered vote / doesn't apply:
-                  THRESHOLD_COD
+                  THRESHOLD_CODE
         """
 
-        votes = self.generate_votes(input_data, by_name=by_name,
+        votes = self.generate_votes(input_data,
                                     missing_strategy=missing_strategy)
-        return votes.combine(method=method, with_confidence=with_confidence,
-                             options=options)
 
-    def generate_votes(self, input_data, by_name=True,
+        result = votes.combine(method=method, options=options, full=full)
+        if full:
+            unused_fields = set(input_data.keys())
+            for _, prediction in enumerate(votes.predictions):
+                unused_fields = unused_fields.intersection( \
+                    set(prediction.get("unused_fields", [])))
+            if not isinstance(result, dict):
+                result = {"prediction": result}
+            result['unused_fields'] = list(unused_fields)
+
+        return result
+
+    def generate_votes(self, input_data,
                        missing_strategy=LAST_PREDICTION):
         """ Generates a MultiVote object that contains the predictions
             made by each of the models.
@@ -141,19 +159,78 @@ class MultiModel(object):
         votes = MultiVote([])
         for order in range(0, len(self.models)):
             model = self.models[order]
-            prediction_info = model.predict(input_data, by_name=by_name,
-                                            with_confidence=True,
-                                            missing_strategy=missing_strategy)
-            prediction, confidence, distribution, instances = prediction_info
-            prediction_row = [prediction, confidence, order,
-                              distribution, instances]
-            votes.append_row(prediction_row)
+            prediction_info = model.predict( \
+                input_data, missing_strategy=missing_strategy, full=True)
+
+            if model.boosting is not None:
+                votes.boosting = True
+                prediction_info.update( \
+                    {"weight": model.boosting.get("weight")})
+                if model.boosting.get("objective_class") is not None:
+                    prediction_info.update( \
+                        {"class": model.boosting.get("objective_class")})
+
+            votes.append(prediction_info)
+
         return votes
 
+    def _generate_votes(self, input_data, missing_strategy=LAST_PREDICTION,
+                        unused_fields=None):
+        """ Generates a MultiVote object that contains the predictions
+            made by each of the models. Please note that this function
+            calls a _predict method which assumes input data has been
+            properly checked against the model fields. Only casting
+            to the correct type will be applied.
+        """
+        votes = MultiVote([])
+        for order in range(0, len(self.models)):
+            model = self.models[order]
+            prediction_info = model._predict( \
+                input_data,
+                missing_strategy=missing_strategy, unused_fields=unused_fields)
+
+            if model.boosting is not None:
+                votes.boosting = True
+                prediction_info.update( \
+                    {"weight": model.boosting.get("weight")})
+                if model.boosting.get("objective_class") is not None:
+                    prediction_info.update( \
+                        {"class": model.boosting.get("objective_class")})
+
+            votes.append(prediction_info)
+
+        return votes
+
+    def generate_votes_distribution(self,
+                                    input_data,
+                                    missing_strategy=LAST_PREDICTION,
+                                    method=PROBABILITY_CODE):
+        votes = []
+        for model in self.models:
+            model.class_names = self.class_names
+            if method == PLURALITY_CODE:
+                prediction_info = [0.0] * len(self.class_names)
+                prediction = model.predict(
+                    input_data,
+                    missing_strategy=missing_strategy,
+                    full=False)
+                prediction_info[self.class_names.index(prediction)] = 1.0
+            else:
+                predict_method = model.predict_confidence \
+                    if method == CONFIDENCE_CODE \
+                    else model.predict_probability
+                prediction_info = predict_method(
+                    input_data,
+                    compact=True,
+                    missing_strategy=missing_strategy)
+            votes.append(prediction_info)
+
+        return MultiVoteList(votes)
+
     def batch_predict(self, input_data_list, output_file_path=None,
-                      by_name=True, reuse=False,
+                      reuse=False,
                       missing_strategy=LAST_PREDICTION, headers=None,
-                      to_file=True):
+                      to_file=True, use_median=False):
         """Makes predictions for a list of input data.
 
            When the to_file argument is set to True, the predictions
@@ -179,6 +256,7 @@ class MultiModel(object):
 
         for model in self.models:
             order += 1
+            out = None
             if to_file:
                 output_file = get_predictions_file_name(model.resource_id,
                                                         output_file_path)
@@ -190,31 +268,35 @@ class MultiModel(object):
                     except IOError:
                         pass
                 try:
-                    predictions_file = csv.writer(open(output_file, 'w', 0),
-                                                  lineterminator="\n")
+                    out = UnicodeWriter(output_file)
                 except IOError:
                     raise Exception("Cannot find %s directory." %
                                     output_file_path)
 
+            if out:
+                out.open_writer()
             for index, input_data in enumerate(input_data_list):
                 if add_headers:
                     input_data = dict(zip(headers, input_data))
                 prediction = model.predict(input_data,
-                                           by_name=by_name,
-                                           with_confidence=True,
-                                           missing_strategy=missing_strategy)
+                                           missing_strategy=missing_strategy,
+                                           full=True)
+                if model.tree.regression:
+                    # if median is to be used, we just replace the prediction
+                    if use_median:
+                        prediction["prediction"] = prediction["median"]
                 if to_file:
-                    if isinstance(prediction[0], basestring):
-                        prediction[0] = prediction[0].encode("utf-8")
-                    predictions_file.writerow(prediction)
+                    prediction = cast_prediction(prediction, to="list",
+                                                 confidence=True,
+                                                 distribution=True,
+                                                 count=True)
+                    out.writerow(prediction)
                 else:
-                    prediction, confidence, distribution, instances = prediction
-                    prediction_row = [prediction, confidence, order,
-                                      distribution, instances]
                     if len(votes) <= index:
                         votes.append(MultiVote([]))
-                    votes[index].append_row(prediction_row)
-
+                    votes[index].append(prediction)
+            if out:
+                out.close_writer()
         if not to_file:
             return votes
 
