@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2013-2015 BigML
+# Copyright 2013-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -54,8 +54,8 @@ RELATIONS = {
 def term_matches(text, forms_list, options):
     """ Counts the number of occurences of the words in forms_list in the text
 
-        The terms in forms_list can either be tokens or full terms. The
-        matching for tokens is contains and for full terms is equals.
+    The terms in forms_list can either be tokens or full terms. The
+    matching for tokens is contains and for full terms is equals.
     """
     token_mode = options.get('token_mode', TM_TOKENS)
     case_sensitive = options.get('case_sensitive', False)
@@ -82,7 +82,7 @@ def full_term_match(text, full_term, case_sensitive):
 
 def get_tokens_flags(case_sensitive):
     """Returns flags for regular expression matching depending on text analysis
-       options
+    options
 
     """
     flags = re.U
@@ -92,7 +92,7 @@ def get_tokens_flags(case_sensitive):
 
 
 def term_matches_tokens(text, forms_list, case_sensitive):
-    """ Counts the number of occurences of the words in forms_list in the text
+    """Counts the number of occurences of the words in forms_list in the text
 
     """
     flags = get_tokens_flags(case_sensitive)
@@ -102,6 +102,30 @@ def term_matches_tokens(text, forms_list, case_sensitive):
     return len(matches)
 
 
+def item_matches(text, item, options):
+    """ Counts the number of occurences of the item in the text
+
+    The matching considers the separator or
+    the separating regular expression.
+    """
+    separator = options.get('separator', ' ')
+    regexp = options.get('separator_regexp')
+    if regexp is None:
+        regexp = ur"%s" % re.escape(separator)
+    return count_items_matches(text, item, regexp)
+
+
+def count_items_matches(text, item, regexp):
+    """ Counts the number of occurences of the item in the text
+
+    """
+    expression = ur'(^|%s)%s($|%s)' % (regexp, item, regexp)
+    pattern = re.compile(expression, flags=re.U)
+    matches = re.findall(pattern, text)
+    return len(matches)
+
+
+
 class Predicate(object):
     """A predicate to be evaluated in a tree's node.
 
@@ -109,9 +133,13 @@ class Predicate(object):
     def __init__(self, operation, field, value, term=None):
         self.operator = operation
         self.missing = False
+
         if self.operator.endswith("*"):
             self.operator = self.operator[0: -1]
             self.missing = True
+        elif operation == 'in' and None in value:
+            self.missing = True
+
         self.field = field
         self.value = value
         self.term = term
@@ -121,6 +149,9 @@ class Predicate(object):
 
         """
         if self.term is not None:
+            # new optype has to be handled in tokens
+            if fields[self.field]['optype'] == 'items':
+                return False
             options = fields[self.field]['term_analysis']
             token_mode = options.get('token_mode', TM_TOKENS)
             if token_mode == TM_FULL_TERM:
@@ -129,13 +160,19 @@ class Predicate(object):
                 return re.match(FULL_TERM_PATTERN, self.term)
         return False
 
-    def to_rule(self, fields, label='name'):
-        """ Builds rule string from a predicate
+    def to_rule(self, fields, label='name', missing=None):
+        """Builds rule string from a predicate
 
         """
-        name = fields[self.field][label]
+        # externally forcing missing to True or False depending on the path
+        if missing is None:
+            missing = self.missing
+        if label is not None:
+            name = fields[self.field][label]
+        else:
+            name = u""
         full_term = self.is_full_term(fields)
-        relation_missing = u" or missing" if self.missing else u""
+        relation_missing = u" or missing" if missing else u""
         if self.term is not None:
             relation_suffix = ''
             if ((self.operator == '<' and self.value <= 1) or
@@ -154,26 +191,37 @@ class Predicate(object):
                                        relation_missing)
         if self.value is None:
             return u"%s %s" % (name,
-                              u"is None" if self.operator == '=' 
-                              else u"is not None")
+                               u"is missing" if self.operator == '='
+                               else u"is not missing")
         return u"%s %s %s%s" % (name,
                                 self.operator,
                                 self.value,
                                 relation_missing)
 
     def to_LISP_rule(self, fields):
-        """ Builds rule string in LISP from a predicate
+        """To be deprecated. See to_lisp_rule
+
+        """
+        self.to_lisp_rule(fields)
+
+    def to_lisp_rule(self, fields):
+        """Builds rule string in LISP from a predicate
 
         """
         if self.term is not None:
-            options = fields[self.field]['term_analysis']
-            case_insensitive = not options.get('case_sensitive', False)
-            case_insensitive = u'true' if case_insensitive else u'false'
-            language = options.get('language')
-            language = u"" if language is None else u" %s" % language
-            return u"(%s (occurrences (f %s) %s %s%s) %s)" % (
-                self.operator, self.field, self.term,
-                case_insensitive, language, self.value)
+            if fields[self.field]['optype'] == 'text':
+                options = fields[self.field]['term_analysis']
+                case_insensitive = not options.get('case_sensitive', False)
+                case_insensitive = u'true' if case_insensitive else u'false'
+                language = options.get('language')
+                language = u"" if language is None else u" %s" % language
+                return u"(%s (occurrences (f %s) %s %s%s) %s)" % (
+                    self.operator, self.field, self.term,
+                    case_insensitive, language, self.value)
+            elif fields[self.field]['optype'] == 'items':
+                return u"(%s (if (contains-items? %s %s) 1 0) %s)" % (
+                    self.operator, self.field, self.term,
+                    self.value)
         if self.value is None:
             negation = u"" if self.operator == "=" else u"not "
             return u"(%s missing? %s)" % (negation, self.field)
@@ -185,27 +233,39 @@ class Predicate(object):
         return rule
 
     def apply(self, input_data, fields):
-        """ Applies the operators defined in the predicate as strings to
-            the provided input data
+        """Applies the operators defined in the predicate as strings to
+        the provided input data
 
         """
 
         # for missing operators
         if input_data.get(self.field) is None:
-            return self.missing or (
-                self.operator == '=' and self.value is None)
+            # text and item fields will treat missing values by following the
+            # doesn't contain branch
+            if self.term is None:
+                return self.missing or (
+                    self.operator == '=' and self.value is None)
         elif self.operator == '!=' and self.value is None:
             return True
 
         if self.term is not None:
-            all_forms = fields[self.field]['summary'].get('term_forms', {})
-            term_forms = all_forms.get(self.term, [])
-            terms = [self.term]
-            terms.extend(term_forms)
-            options = fields[self.field]['term_analysis']
-            return apply(OPERATOR[self.operator],
-                         [term_matches(input_data[self.field], terms, options),
-                          self.value])
+            if fields[self.field]['optype'] == 'text':
+                all_forms = fields[self.field]['summary'].get('term_forms', {})
+                term_forms = all_forms.get(self.term, [])
+                terms = [self.term]
+                terms.extend(term_forms)
+                options = fields[self.field]['term_analysis']
+                return apply(OPERATOR[self.operator],
+                             [term_matches(input_data.get(self.field, ""),
+                                           terms, options),
+                              self.value])
+            else:
+                # new items optype
+                options = fields[self.field]['item_analysis']
+                return apply(OPERATOR[self.operator],
+                             [item_matches(input_data.get(self.field, ""),
+                                           self.term, options),
+                              self.value])
         if self.operator == "in":
             return apply(OPERATOR[self.operator],
                          [self.value,
