@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -30,7 +30,7 @@ class Predicates(object):
     def __init__(self, predicates_list):
         self.predicates = []
         for predicate in predicates_list:
-            if predicate == True:
+            if predicate is True:
                 self.predicates.append(True)
             else:
                 self.predicates.append(
