@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,14 +17,21 @@
 
 """Base class for sources' REST calls
 
+   https://bigml.com/api/sources
+
 """
 
 import sys
 import os
-import requests
-import ssl
-
-from threading import Thread
+import urllib2
+import numbers
+try:
+    #added to allow GAE to work
+    from google.appengine.api import urlfetch
+    GAE_ENABLED = True
+except ImportError:
+    GAE_ENABLED = False
+    import ssl
 
 try:
     import simplejson as json
@@ -32,33 +39,41 @@ except ImportError:
     import json
 
 
-import urllib2
-from poster.encode import multipart_encode, MultipartParam
+from threading import Thread
+
 
 PYTHON_2_7_9 = len(urllib2.urlopen.__defaults__) > 2
+PYTHON_2 = sys.version_info < (3, 0)
 
+if PYTHON_2:
+    from poster.encode import multipart_encode, MultipartParam
 if PYTHON_2_7_9:
     from bigml.sslposter import StreamingHTTPSHandler, register_openers
-else:
+elif PYTHON_2:
     from poster.streaminghttp import StreamingHTTPSHandler, register_openers
+else:
+    from requests_toolbelt import MultipartEncoder
+    import mimetypes
 
 from bigml.util import (localize, clear_console_line, reset_console_line,
                         console_log, is_url)
-from bigml.bigmlconnection import BigMLConnection
 from bigml.bigmlconnection import (
     HTTP_CREATED, HTTP_ACCEPTED, HTTP_BAD_REQUEST,
     HTTP_UNAUTHORIZED, HTTP_PAYMENT_REQUIRED, HTTP_NOT_FOUND,
-    HTTP_TOO_MANY_REQUESTS,
-    HTTP_INTERNAL_SERVER_ERROR)
-from bigml.resourcehandler import (check_resource_type, get_resource,
-                                   resource_is_ready, check_resource,
+    HTTP_TOO_MANY_REQUESTS, HTTP_FORBIDDEN,
+    HTTP_INTERNAL_SERVER_ERROR, GAE_ENABLED, SEND_JSON)
+from bigml.bigmlconnection import json_load
+from bigml.resourcehandler import (check_resource_type,
+                                   resource_is_ready,
                                    get_source_id)
-from bigml.resourcehandler import SOURCE_RE, SOURCE_PATH, UPLOADING, LOGGER
-from bigml.resourcehandler import ResourceHandler
-
-
-register_openers()
+from bigml.constants import SOURCE_PATH, UPLOADING
+from bigml.resourcehandler import ResourceHandler, LOGGER
 
+if PYTHON_2:
+    register_openers()
+else:
+    import requests
+    from bigml.util import maybe_save
 
 class SourceHandler(ResourceHandler):
 
@@ -85,6 +100,19 @@ class SourceHandler(ResourceHandler):
         if args is not None:
             create_args.update(args)
         create_args.update({"remote": url})
+        create_args = self._add_project(create_args)
+        body = json.dumps(create_args)
+        return self._create(self.source_url, body)
+
+    def _create_connector_source(self, connector, args=None):
+        """Creates a new source using an external connector
+
+        """
+        create_args = {}
+        if args is not None:
+            create_args.update(args)
+        create_args.update({"external_data": connector})
+        create_args = self._add_project(create_args)
         body = json.dumps(create_args)
         return self._create(self.source_url, body)
 
@@ -97,11 +125,12 @@ class SourceHandler(ResourceHandler):
         create_args = {}
         if args is not None:
             create_args.update(args)
+        create_args = self._add_project(create_args)
 
         # some basic validation
-        if (not isinstance(src_obj, list) or
-            (not all([isinstance(row, dict) for row in src_obj]) and
-             not all([isinstance(row, list) for row in src_obj]))):
+        if (not isinstance(src_obj, list) or (
+                not all([isinstance(row, dict) for row in src_obj]) and
+                not all([isinstance(row, list) for row in src_obj]))):
             raise TypeError(
                 'ERROR: inline source must be a list of dicts or a '
                 'list of lists')
@@ -110,73 +139,6 @@ class SourceHandler(ResourceHandler):
         body = json.dumps(create_args)
         return self._create(self.source_url, body)
 
-    def _create_local_source(self, file_name, args=None):
-        """Creates a new source using a local file.
-
-        This function is now DEPRECATED as "requests" do not stream the file
-        content and that limited the size of local files to a small number of
-        GBs.
-
-        """
-        create_args = {}
-        if args is not None:
-            create_args.update(args)
-
-        if 'source_parser' in create_args:
-            create_args['source_parser'] = json.dumps(
-                create_args['source_parser'])
-
-        code = HTTP_INTERNAL_SERVER_ERROR
-        resource_id = None
-        location = None
-        resource = None
-        error = {
-            "status": {
-                "code": code,
-                "message": "The resource couldn't be created"}}
-
-        try:
-            files = {os.path.basename(file_name): open(file_name, "rb")}
-        except IOError:
-            raise IOError("ERROR: cannot read training set")
-
-        try:
-            response = requests.post(self.source_url + self.auth,
-                                     files=files,
-                                     data=create_args, verify=self.verify)
-
-            code = response.status_code
-            if code == HTTP_CREATED:
-                location = response.headers['location']
-                resource = json.loads(response.content, 'utf-8')
-                resource_id = resource['resource']
-                error = None
-            elif code in [HTTP_BAD_REQUEST,
-                          HTTP_UNAUTHORIZED,
-                          HTTP_PAYMENT_REQUIRED,
-                          HTTP_NOT_FOUND,
-                          HTTP_TOO_MANY_REQUESTS]:
-                error = json.loads(response.content, 'utf-8')
-            else:
-                LOGGER.error("Unexpected error (%s)", code)
-                code = HTTP_INTERNAL_SERVER_ERROR
-
-        except ValueError:
-            LOGGER.error("Malformed response")
-        except requests.ConnectionError, exc:
-            LOGGER.error("Connection error: %s", str(exc))
-        except requests.Timeout:
-            LOGGER.error("Request timed out")
-        except requests.RequestException:
-            LOGGER.error("Ambiguous exception occurred")
-
-        return {
-            'code': code,
-            'resource': resource_id,
-            'location': location,
-            'object': resource,
-            'error': error}
-
     def _upload_source(self, args, source, out=sys.stdout):
         """Uploads a source asynchronously.
 
@@ -200,7 +162,7 @@ class SourceHandler(ResourceHandler):
         source['object'] = resource['object']
         source['error'] = resource['error']
 
-    def _stream_source(self, file_name, args=None, async=False,
+    def _stream_source(self, file_name, args=None, async_load=False,
                        progress_bar=False, out=sys.stdout):
         """Creates a new source.
 
@@ -212,7 +174,7 @@ class SourceHandler(ResourceHandler):
             """
             pct = 100 - ((total - current) * 100) / (total)
             console_log("Uploaded %s out of %s bytes [%s%%]" % (
-                localize(current), localize(total), pct))
+                localize(current), localize(total), pct), reset=True)
         create_args = {}
         if args is not None:
             create_args.update(args)
@@ -231,15 +193,14 @@ class SourceHandler(ResourceHandler):
                                     open(file_name, "rb")})
             else:
                 create_args = create_args.items()
-                name = '<none>'
+                name = 'Stdin input'
                 create_args.append(MultipartParam(name, filename=name,
                                                   fileobj=file_name))
-
         except IOError, exception:
             raise IOError("Error: cannot read training set. %s" %
                           str(exception))
 
-        if async:
+        if async_load:
             source = {
                 'code': HTTP_ACCEPTED,
                 'resource': resource_id,
@@ -271,56 +232,98 @@ class SourceHandler(ResourceHandler):
                 "code": code,
                 "message": "The resource couldn't be created"}}
 
+        if args is None:
+            args = {}
+        args = self._add_project(args, True)
+
         if progress_bar and callback is not None:
             body, headers = multipart_encode(args, cb=callback)
         else:
             body, headers = multipart_encode(args)
 
-        request = urllib2.Request(self.source_url + self.auth, body, headers)
+        url = self._add_credentials(self.source_url)
 
-        try:
-            # try using the new SSL checking in python 2.7.9
+        if GAE_ENABLED:
             try:
-                if not self.verify and PYTHON_2_7_9:
-                    context = ssl.create_default_context(
-                        ssl.Purpose.CLIENT_AUTH)
-                    context.verify_mode = ssl.CERT_NONE
-                    https_handler = StreamingHTTPSHandler(context=context)
-                    opener = urllib2.build_opener(https_handler)
-                    urllib2.install_opener(opener)
+                response = urlfetch.fetch(url=url,
+                                          payload="".join(body),
+                                          method=urlfetch.POST,
+                                          headers=headers)
+                code = response.status_code
+                content = response.content
+                if code in [HTTP_CREATED]:
+                    if 'location' in response.headers:
+                        location = response.headers['location']
+                    resource = json_load(response.content)
+                    resource_id = resource['resource']
+                    error = {}
+                elif code in [HTTP_BAD_REQUEST,
+                              HTTP_UNAUTHORIZED,
+                              HTTP_PAYMENT_REQUIRED,
+                              HTTP_FORBIDDEN,
+                              HTTP_NOT_FOUND,
+                              HTTP_TOO_MANY_REQUESTS]:
+                    error = json_load(response.content)
+                    LOGGER.error(self.error_message(error, method='create'))
+                elif code != HTTP_ACCEPTED:
+                    LOGGER.error("Unexpected error (%s)", code)
+                    code = HTTP_INTERNAL_SERVER_ERROR
+            except urlfetch.Error, exception:
+                LOGGER.error("Error establishing connection: %s",
+                             str(exception))
+        else:
+            try:
+                request = urllib2.Request(url,
+                                          body, headers)
+                # try using the new SSL checking in python 2.7.9
+                try:
+                    if not self.verify and PYTHON_2_7_9:
+                        context = ssl.create_default_context(
+                            ssl.Purpose.CLIENT_AUTH)
+                        context.verify_mode = ssl.CERT_NONE
+                        https_handler = StreamingHTTPSHandler(context=context)
+                        opener = urllib2.build_opener(https_handler)
+                        urllib2.install_opener(opener)
+                        response = urllib2.urlopen(request)
+                    else:
+                        response = urllib2.urlopen(request)
+                except AttributeError:
                     response = urllib2.urlopen(request)
+                clear_console_line(out=out)
+                reset_console_line(out=out)
+                code = response.getcode()
+                if code == HTTP_CREATED:
+                    location = response.headers['location']
+                    content = response.read()
+                    resource = json_load(content)
+                    resource_id = resource['resource']
+                    error = {}
+                    if self.debug:
+                        LOGGER.debug("Data: %s", body)
+                        LOGGER.debug("Response: %s", content)
+            except ValueError:
+                LOGGER.error("Malformed response.")
+            except urllib2.HTTPError, exception:
+                code = exception.code
+                if code in [HTTP_BAD_REQUEST,
+                            HTTP_UNAUTHORIZED,
+                            HTTP_PAYMENT_REQUIRED,
+                            HTTP_NOT_FOUND,
+                            HTTP_TOO_MANY_REQUESTS]:
+                    content = exception.read()
+                    error = json_load(content)
+                    if self.debug:
+                        LOGGER.debug("Data: %s", body)
+                        LOGGER.debug("Response: %s", content)
+                    LOGGER.error(self.error_message(error, method='create'))
                 else:
-                    response = urllib2.urlopen(request)
-            except AttributeError:
-                response = urllib2.urlopen(request)
-            clear_console_line(out=out)
-            reset_console_line(out=out)
-            code = response.getcode()
-            if code == HTTP_CREATED:
-                location = response.headers['location']
-                content = response.read()
-                resource = json.loads(content, 'utf-8')
-                resource_id = resource['resource']
-                error = {}
-        except ValueError:
-            LOGGER.error("Malformed response")
-        except urllib2.HTTPError, exception:
-            code = exception.code
-            if code in [HTTP_BAD_REQUEST,
-                        HTTP_UNAUTHORIZED,
-                        HTTP_PAYMENT_REQUIRED,
-                        HTTP_NOT_FOUND,
-                        HTTP_TOO_MANY_REQUESTS]:
-                content = exception.read()
-                error = json.loads(content, 'utf-8')
-                LOGGER.error(self.error_message(error, method='create'))
-            else:
-                LOGGER.error("Unexpected error (%s)", code)
-                code = HTTP_INTERNAL_SERVER_ERROR
+                    LOGGER.error("Unexpected error (%s)", code)
+                    code = HTTP_INTERNAL_SERVER_ERROR
 
-        except urllib2.URLError, exception:
-            LOGGER.error("Error establishing connection: %s", str(exception))
-            error = exception.args
+            except urllib2.URLError, exception:
+                LOGGER.error("Error establishing connection: %s",
+                             str(exception))
+                error = exception.args
         return {
             'code': code,
             'resource': resource_id,
@@ -328,7 +331,105 @@ class SourceHandler(ResourceHandler):
             'object': resource,
             'error': error}
 
-    def create_source(self, path=None, args=None, async=False,
+    def _create_local_source(self, file_name, args=None):
+        """Creates a new source using a local file.
+
+        This function is only used from Python 3. No async-prepared.
+
+        """
+        create_args = {}
+        if args is not None:
+            create_args.update(args)
+
+        for key, value in create_args.items():
+            if value is not None and (isinstance(value, list) or
+                                      isinstance(value, dict)):
+                create_args[key] = json.dumps(value)
+            elif value is not None and isinstance(value, numbers.Number):
+                # the multipart encoder only accepts strings and files
+                create_args[key] = str(value)
+
+
+        code = HTTP_INTERNAL_SERVER_ERROR
+        resource_id = None
+        location = None
+        resource = None
+        error = {
+            "status": {
+                "code": code,
+                "message": "The resource couldn't be created"}}
+
+        try:
+
+            if isinstance(file_name, basestring):
+                name = os.path.basename(file_name)
+                file_handler = open(file_name, "rb")
+            else:
+                name = 'Stdin input'
+                file_handler = file_name
+        except IOError:
+            sys.exit("ERROR: cannot read training set")
+
+        url = self._add_credentials(self.source_url)
+        create_args = self._add_project(create_args, True)
+        if GAE_ENABLED:
+            try:
+                req_options = {
+                    'url': url,
+                    'method': urlfetch.POST,
+                    'headers': SEND_JSON,
+                    'data': create_args,
+                    'files': {name: file_handler},
+                    'validate_certificate': self.verify
+                }
+                response = urlfetch.fetch(**req_options)
+            except urlfetch.Error, exception:
+                LOGGER.error("HTTP request error: %s",
+                             str(exception))
+                return maybe_save(resource_id, self.storage, code,
+                                  location, resource, error)
+        else:
+            try:
+                files = {"file": (name,
+                                  file_handler,
+                                  mimetypes.guess_type(name)[0])}
+                files.update(create_args)
+                multipart = MultipartEncoder(fields=files)
+                response = requests.post( \
+                    url,
+                    headers={'Content-Type': multipart.content_type},
+                    data=multipart, verify=self.verify)
+            except (requests.ConnectionError,
+                    requests.Timeout,
+                    requests.RequestException), exc:
+                LOGGER.error("HTTP request error: %s", str(exc))
+                code = HTTP_INTERNAL_SERVER_ERROR
+                return maybe_save(resource_id, self.storage, code,
+                                  location, resource, error)
+        try:
+            code = response.status_code
+            if code == HTTP_CREATED:
+                location = response.headers['location']
+                resource = json_load(response.content)
+                resource_id = resource['resource']
+                error = None
+            elif code in [HTTP_BAD_REQUEST,
+                          HTTP_UNAUTHORIZED,
+                          HTTP_PAYMENT_REQUIRED,
+                          HTTP_NOT_FOUND,
+                          HTTP_TOO_MANY_REQUESTS]:
+                error = json_load(response.content)
+            else:
+                LOGGER.error("Unexpected error (%s)" % code)
+                code = HTTP_INTERNAL_SERVER_ERROR
+
+        except ValueError:
+            LOGGER.error("Malformed response")
+
+        return maybe_save(resource_id, self.storage, code,
+                          location, resource, error)
+
+    def create_source(self, path=None, args=None, async_load=False,
                       progress_bar=False, out=sys.stdout):
         """Creates a new source.
 
@@ -343,9 +444,14 @@ class SourceHandler(ResourceHandler):
             return self._create_remote_source(path, args=args)
         elif isinstance(path, list):
             return self._create_inline_source(path, args=args)
-        else:
-            return self._stream_source(file_name=path, args=args, async=async,
+        elif isinstance(path, dict):
+            return self._create_connector_source(path, args=args)
+        elif PYTHON_2:
+            return self._stream_source(file_name=path, args=args,
+                                       async_load=async_load,
                                        progress_bar=progress_bar, out=out)
+        else:
+            return self._create_local_source(file_name=path, args=args)
 
     def get_source(self, source, query_string=''):
         """Retrieves a remote source.
